/********************************************************************/
/*                              Poly                                */
/*                                                                  */
/*   Luc Maranget, projet Moscova, Inria Rocquencourt               */
/*                                                                  */
/* Copyright 2005 Institut National de Recherche en Informatique et */
/* en Automatique.  All rights reserved.  This file is distributed  */
/* under the terms of the Q Public License version 1.0.             */ 
/********************************************************************/

#define NMAX 30

int n = 14;     /* rang */
int idamier,jdamier;     /* taille du damier  */
int icentre,jcentre;     /* coordonne du centre de rotation (pour ii) */

typedef unsigned long small_counter;

#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include<unistd.h>
#include<string.h>
#include<time.h>
#include"count.h"

/* Connexite dans l'interieur du ring */

typedef enum ring_pos {dehors, dedans, dessus} ring_pos;

typedef enum statut {libre, plein, start, end, voisin} statut;

typedef struct point {
  statut stat;
  int i,j;
  struct point *next;
  struct point *voisins[5];
  int field;
  struct point *image;
  struct point *image2;
  ring_pos pos;
} point;

int nimages=0;

/* damier */
#define IMAX NMAX
#define JMAX (3*NMAX)/2

point damier[IMAX][JMAX];

int is_plein(point *p)
{
  return (p->stat == plein || p->stat == end || p->stat == start);
}


void set_pos(point *p,ring_pos s)
{
  int i;
  
  for (i = nimages ; i >= 0 ; i--) {
    p->pos = s;
    p = p->image ;
  }
}

void set_status(point *p,statut s)
{
  int i;

  for (i=nimages ; i >= 0 ; i--) {
    p->stat = s;
    p = p->image;
  }
}

int is_neb(point *p,point *q)
{
  point **v;

  for (v = p->voisins ; *v ; v++)
    if (*v == q) return 1;
  return 0;
}

int verbose_ring = 0;
int verbose_pol = 0;
int ring_only = 0;

int double_check = 0;


void print_sol()
{
  int i,j;
  point *r;

  for (j=jdamier-1;j >= 0;j--) {
    for (i=0;i< idamier;i++) {
      r = &damier[i][j];
      putchar(r->stat==libre?'.':\
              (r->stat==plein?'#':\
               (r->stat==voisin?(r->field?'X':'x'):'*')));
    }
    putchar('\n');
  }
  putchar('\n');
}

/* Precalcul des voisins (les vrais) et des images */

int check_next(int i,int j)
{
  if ((0 <= i) && (i < idamier) && (0 <= j) && (j < jdamier)) {
    return 1;
  } else {
    return 0;
  }
}

void set_nexts(int i,int j) {
  int k=0;
  point *r = &damier[i][j];

  
  if (check_next(i+1,j)) {r->voisins[k++] = &damier[i+1][j];}
  if (check_next(i,j+1)) {r->voisins[k++] = &damier[i][j+1];}
  if (check_next(i-1,j)) {r->voisins[k++] = &damier[i-1][j];}
  if (check_next(i,j-1)) {r->voisins[k++] = &damier[i][j-1];}
  r->voisins[k] = 0;
}


void init_damierii(void)
{
  int i,j;

  if (verbose_ring || verbose_pol || double_check)
    printf("--------- RII ---------\n");
  nimages = 1;
  if (n%2 == 0) {
    idamier = jdamier = n - 5;
    icentre = jcentre = n/2-3;
  } else {
    idamier = jdamier = n;
    icentre = jcentre = (n-1)/2;
  }
  for (i=0;i< idamier;i++) {
    for (j=0;j< jdamier;j++) {
      damier[i][j].i = i;
      damier[i][j].j = j;
      set_nexts(i,j);
      damier[i][j].image = &damier[2*icentre-i][2*jcentre-j];
      damier[i][j].image2 = &damier[icentre + jcentre - j]\
        [i + jcentre - icentre];
    }
  }
  for (i=icentre ; i < idamier ; i++)
    damier[i][jcentre].field = 1;
  for (i=0;i<idamier;i++) {
    for (j=jcentre+1; j < jdamier;j++) {
      damier[i][j].field = 1;
    }
  }
}

void init_damier2i(void)
{
  int i,j;

  if (verbose_ring || verbose_pol || double_check)
    printf("--------- R2I ---------\n");

  nimages = 3;

  if (n % 4 == 1) {
    idamier = jdamier = 1 + (n-1)/2;
    icentre = jcentre = (n-1)/4;
  } else { /* n % 4 == 0 */
    idamier = jdamier = n/2 - 1;
    icentre = jcentre = n/4 - 1;
  }

  for (i=0;i< idamier;i++) {
    for (j=0;j< jdamier;j++) {
      damier[i][j].field = 0;
      damier[i][j].i = i;
      damier[i][j].j = j;
      set_nexts(i,j);
      damier[i][j].image = &damier[icentre + jcentre - j]\
        [i + jcentre - icentre];
    }
  }
  damier[icentre][jcentre].field = 1;
  for (i = icentre+1 ; i < idamier ; i++)
    for (j = jcentre ; j < jdamier ; j++)
      damier[i][j].field = 1;
}

void init_damierxx(void)
{
  int i,j;

  if (verbose_ring || verbose_pol || double_check)
    printf("--------- RXX ---------\n");

  nimages = 1;
  idamier = jdamier = n-2;
  icentre = jcentre = (n-2)/2; /* upper-right square wrt to center */
  for (i=0;i< idamier;i++) {
    for (j=0;j< jdamier;j++) {
      damier[i][j].i = i;
      damier[i][j].j = j;
      set_nexts(i,j);
      damier[i][j].image = &damier[idamier-i-1][jdamier-j-1];
      damier[i][j].image2 = &damier[icentre + jcentre - 1 -j]\
        [i+jcentre - icentre];
    }
  }
  for (i=0;i<idamier;i++) {
    for (j=0; j < jcentre ; j++) {
      damier[i][j].field = 0;
    }
    for (j=jcentre; j < jdamier;j++) {
      damier[i][j].field = 1;
    }
  }
}

void init_damier2x(void)
{
  int i,j;

  if (verbose_ring || verbose_pol || double_check)
    printf("--------- R2X ---------\n");

  nimages = 3;
  idamier = jdamier = n/2;
  icentre = jcentre = n/4; /* upper-right square wrt to center */
  for (i=0;i< idamier;i++) {
    for (j=0;j< jdamier;j++) {
      damier[i][j].i = i;
      damier[i][j].j = j;
      damier[i][j].field = 0;
      set_nexts(i,j);
      damier[i][j].image = &damier[icentre + jcentre - 1 -j]\
        [i+jcentre - icentre];
    }
  }
  for (i = icentre ; i < idamier ; i++)
    for(j = jcentre ; j < jdamier ; j++)
      damier[i][j].field = 1;
}
void init_damierxi(void)
{
  int i,j;

  if (verbose_ring || verbose_pol)
    printf("--------- RXI ---------\n");

  nimages = 1;
  idamier = n;
  jdamier = n-1;
  icentre = n/2;
  jcentre = (n-2)/2;

  for (i=0;i< idamier;i++) {
    for (j=0;j< jdamier;j++) {
      damier[i][j].i = i;
      damier[i][j].j = j;
      set_nexts(i,j);
      damier[i][j].image = &damier[idamier-i-1][jdamier-j-1];
    }
  }
  for (j=0 ; j < jcentre ; j++)
    for (i=0 ; i < idamier ; i++)
      damier[i][j].field = 0;
  for (i=0 ; i < icentre ; i++)
    damier[i][jcentre].field = 0;
  for (i=icentre ; i < idamier ; i++)
    damier[i][jcentre].field = 1;
  for (j = jcentre+1 ; j < jdamier ; j++)
    for (i = 0 ; i < idamier ; i++)
      damier[i][j].field = 1;
}

void init_damiervi(void)
{
  int i,j;


  if (verbose_ring || verbose_pol)
    printf("--------- VI ---------\n");

  nimages = 1;
  if (n == 1) {
    idamier = jdamier = 1;
    icentre = jcentre = 0;
  } else if (n == 2) {
    idamier = 1;
    jdamier = 2;
    icentre = jcentre = 0;
  } else if (n % 2 == 1) {
    idamier = n;
    jdamier = 3*(n-1)/2;
    icentre = (n-1)/2;
    jcentre = (n-3)/2;
  } else {
    idamier = n-1;
    jdamier = (3 * n-4)/2;
    icentre = (n-2)/2;
    jcentre = (n-4)/2;
  }

  for (i=0;i< idamier;i++) {
    for (j=0;j< jdamier;j++) {
      damier[i][j].i = i;
      damier[i][j].j = j;
      set_nexts(i,j);
      damier[i][j].image = &damier[idamier-i-1][j];
    }
  }

  for (i = 0 ; i < icentre ; i++)
    for (j = 0 ; j < idamier ; j++)
      damier[i][j].field = 0;
  for (i = icentre ; i < idamier ; i++)
    for (j = 0 ; j < jdamier ; j++)
      damier[i][j].field = 1;
}

void init_damiervx(void)
{
  int i,j;


  if (verbose_ring || verbose_pol)
    printf("--------- VX ---------\n");

  nimages = 1;
  if (n == 2) {
    idamier = 2;
    jdamier = 1;
    icentre = 1;
    jcentre = 0;
  } else {
    idamier = n;
    jdamier = n-2;
    icentre = n/2;
    jcentre = (n-4)/2;
  }


  for (i=0;i< idamier;i++) {
    for (j=0;j< jdamier;j++) {
      damier[i][j].i = i;
      damier[i][j].j = j;
      set_nexts(i,j);
      damier[i][j].image = &damier[idamier-i-1][j];
    }
  }
  for (i = 0 ; i < icentre ; i++)
    for (j = 0 ; j < idamier ; j++)
      damier[i][j].field = 0;
  for (i = icentre ; i < idamier ; i++)
    for (j = 0 ; j < jdamier ; j++)
      damier[i][j].field = 1;
}

void init_damierd(void)
{
  int i,j;

  if (verbose_ring || verbose_pol)
    printf("--------- D ---------\n");

  nimages = 1;

  if (n % 2 == 1) {
    idamier = jdamier = n;
    icentre = jcentre = (n-1)/2;
  } else {
    idamier = jdamier = n-1;
    icentre = jcentre = (n-2)/2;
  }

  for (i=0;i< idamier;i++) {
    for (j=0;j< jdamier;j++) {
      damier[i][j].i = i;
      damier[i][j].j = j;
      set_nexts(i,j);
      damier[i][j].image = &damier[j][i];
    }
  } 
  for (i = 0 ; i < idamier ; i++) {
    for (j = 0 ; j < i ; j++)
      damier[i][j].field = 0;
    for (j = i ; j < jdamier ; j++)
      damier[i][j].field = 1;
  }

}

void patch_for_ring(point *pstart)
{
  int j;

  for(j = 0 ; j < pstart->j ; j++)
    set_status(&damier[pstart->i][j],voisin);
  if (pstart->i+1< idamier)
    for (j = 0 ; j < jdamier ; j++)
      set_status(&damier[pstart->i+1][j],voisin);
}


void unpatch_for_ring(point *pstart)
{
  int j;

  for(j = 0 ; j < pstart->j ; j++)
    set_status(&damier[pstart->i][j],libre);
  if (pstart->i+1< idamier)
    for (j = 0 ; j < jdamier ; j++)
      set_status(&damier[pstart->i+1][j],libre);
}

void patch_vsym(point *p)
{
  int j;

  for (j = 0 ; j < p->j ; j++)
    set_status(&damier[p->i][j],voisin);
}

void unpatch_vsym(point *p)
{
  int j;

  for (j = 0 ; j < p->j ; j++)
    set_status(&damier[p->i][j],libre);
}

void patch_dsym(point *p)
{
  int j;

  for (j = 0 ; j < p->j ; j++)
    set_status(&damier[j][j],voisin);
}

void unpatch_dsym(point *p)
{
  int j;

  for (j = 0 ; j < p->j ; j++)
    set_status(&damier[j][j],libre);
}

void start_damier(point *p)
{
  int i;

  p->stat = start;
  p = p->image;
  p->stat = end;
  for (i = 0 ; i < nimages-1 ; i++) {
    p = p->image;
    p->stat = plein;
  }
}

void restore_damier(point *p)
{
  set_status(p,libre);
}

/* center inside cell */
int seen_end;

int nvois(point *p)
{
  point **v;
  int r = 0;

  seen_end=0;
  for (v =  p->voisins ; *v ; v++)
    switch ((*v)->stat) {
    case end:
      seen_end = 1;
    case plein:
    case start:
      r++;
      break;
    default:
      ;
    }
  return r;
}


point *see[IMAX];

int add_see(point *p,int k)
{
  see[k++] = p;
  return k;
}

int mark[NMAX][NMAX];

int check_ring(point *p, int k)
{
  if (mark[p->i][p->j])
    return k;

  mark[p->i][p->j] = 1;

  if (p->pos == dessus)
    return add_see(p,k);
  else {
    point **v;

    for (v = p->voisins ; *v ; v++) {
      if (is_plein(*v))
        k = check_ring(*v,k);
    }
    return k;
  }
}

void erase_mark(point *p)
{
  point **v;

  if (!mark[p->i][p->j])
    return;
  
  mark[p->i][p->j] = 0;
  for (v = p->voisins ; *v ; v++)
    erase_mark(*v);
}

int keep_ring(point *p)
{
  int k=0;

  k = check_ring(p,k);
  erase_mark(p);

  if (k <= 1)
    return 1;
  if (k == 2)
    return is_neb(see[0],see[1]);
  return 0;
}

point *new_nebs(point *p,point *untried)
{
  point **v,*me;

  for (v = p->voisins ; *v ; v++) {
    me = *v;
    if (me->stat == libre) {
      set_status(me,voisin);
      while (!me->field)
        me = me->image;
      me->next = untried;
      untried = me;
    }
  }
  return untried;
}

void del_nebs(point *untried,point *old)
{
  for ( ; untried != old ; untried = untried->next)
    set_status(untried,libre);
}


int value(point *p)
{
  if (p->image == p)
    return 1;
  else
    return nimages+1;
}

small_counter search(int k,point *untried)
{

  if (k <= 0) {
    if (k == 0) {
      if (verbose_pol > 0) {
        printf("One solution\n");  
        print_sol();
      }
      if (double_check) {
        if (nimages == 3) {
          printf("-------------------\n");
          print_sol();
        } else {
          int i,j;
          
          for (i = 0 ; i < idamier ; i++)
            for (j = 0 ; j < jdamier ; j++)
              if (is_plein(&damier[i][j]) &&\
                  !is_plein(damier[i][j].image2))
                goto no;
          printf("-------------------\n");
          print_sol();
        no:
          ;
        }
      }
      return 1;
    } else
      return 0;
  } else {
    point *me,*news;
    int r=0;

    if (verbose_pol > 1) {
      printf("trying... (%d)\n",n-k);
      print_sol();
    }
    while (untried) {
      me = untried;
      untried = untried->next;
      set_status(me,plein);
      if (me->pos == dedans && !keep_ring(me)) {
        set_status(me,voisin);
        continue;
      }
      news = new_nebs(me,untried);
      r += search(k-value(me),news);
      set_status(me,voisin);
      del_nebs(news,untried);
    }
    return r;
  }
}

typedef struct cell {
  point *car;
  struct cell *cdr;
} cell;

void flood(point *p,ring_pos pos)
{
  if (p->pos == dessus || p->pos == pos)
    return;
  else {
    point **v;

    p->pos = pos;
    for (v = p->voisins ; *v ; v++)
      flood(*v,pos);
  }
  return;
}

small_counter from_ring(int k,point *p,point *pstart,cell *prev)
{
  small_counter r;
  point *untried = NULL;
  cell *t;

  unpatch_for_ring(pstart);
  if (verbose_ring > 0) {
    printf("One ring of size: %d\n",n-k);  
    print_sol();
  }

  if (!ring_only) {
    int i,j;

    for (i = 0 ; i < idamier ; i++)
      for (j = 0 ; j < jdamier ; j++)
        damier[i][j].pos = dedans;

    for (t = prev ; t ; t = t->cdr) {
      set_pos(t->car,dessus);
    }
    flood(&damier[0][0],dehors);
    for (t = prev ; t ; t = t->cdr)
      untried = new_nebs(t->car,untried);
    
    if (verbose_ring > 0) {
      printf("Ring with neibourghs\n");  
      print_sol();
    }

    r = search(k,untried);

    for (i = 0 ; i < idamier ; i++)
      for (j = 0 ; j < jdamier ; j++)
        damier[i][j].pos = dehors;
    del_nebs(untried,NULL);
  } else
    r = 1;

  patch_for_ring(pstart);
  return r;
}


static int distance(point * p, point *q)
{
  return abs(p->i - q->i) + abs(p->j - q->j);
}

unsigned to_end(point *p, point *pstart)
{
  int k;
  int d;
  int dmin=idamier+jdamier;

  for (k = 0 ; k < nimages ; k++) {
    pstart = pstart->image;
    if ((d = distance(p,pstart)) < dmin)
      dmin = d;
  }
  return (nimages+1)*(dmin-1);
}


small_counter inc_ring(int k,point *p,point *pstart,cell *prev)
{
  point **v,*q;
  small_counter r=0;
  cell here;
  int d;


  if (verbose_ring > 1 ) {
    printf("Current size: %d\n",n-k);
    print_sol();
  }
  d = to_end(p,pstart);

  if (d == 0)
    return from_ring(k,p,pstart,prev);
  else if (d > k) return r; /* echec */

  for (v = p->voisins ; *v ; v++) {
    q = *v;
    if (q->stat == libre) {
      set_status(q,plein); /* just to see what would happen */
      if (nvois(q) == 1) {
        here.cdr = prev;
        here.car = q;
        r += inc_ring(k-nimages-1,q,pstart,&here);
      } else if (seen_end && nvois(q) == 2) {
        here.cdr = prev;
        here.car = q;
        r += inc_ring(k-nimages-1,q,pstart,&here);
      }
      set_status(q,libre);
      if (pstart == p)
        return r;
    }
  }
  return r;
}

small_counter ringii(void)
{
  int ki,kj;
  small_counter r=0;
  point *pstart;
  cell top;
  int jlim = (nimages==1?jdamier:jcentre+1);

  if (n%2 == 0) {
    if (n < 8) return r;
    set_status(&damier[icentre][jcentre],voisin);
    for (ki = icentre+1 ; ki < idamier ; ki++)
        for (kj = 0 ; kj < jlim ; kj++) {
        pstart = &damier[ki][kj];
        if (pstart->i < pstart->image->i)
          pstart = pstart->image;
        start_damier(pstart);
        patch_for_ring(pstart);
        top.car = pstart;
        top.cdr = NULL;
        r += inc_ring(n-nimages-1,pstart,pstart,&top);
        unpatch_for_ring(pstart);
        restore_damier(pstart);
      }
    set_status(&damier[icentre][jcentre],voisin);
  } else {
    point *p = &damier[icentre][jcentre];
    point *untried;

    p->next = NULL;
    p->stat = plein; /* degenerated ring */
    untried = new_nebs(p,NULL);
    r = search(n-1,untried);
    del_nebs(untried,NULL);
    p->stat = libre;
  }
  return r;
}

small_counter ringxx(void)
{
  int ki,kj;
  small_counter r=0;
  point *pstart;
  cell top;
  int jlim = (nimages==1?jdamier:jcentre);

  if (n % 2 == 1 || n < 4)
    return r;

  for (ki = icentre ; ki < idamier ; ki++)
    for (kj = 0 ; kj < jlim ; kj++) {
      pstart = &damier[ki][kj];
      if (pstart->i < pstart->image->i)
        pstart = pstart->image;
      start_damier(pstart);
      patch_for_ring(pstart);
      top.car = pstart;
      top.cdr = NULL;
      r += inc_ring(n-nimages-1,pstart,pstart,&top);
      unpatch_for_ring(pstart);
      restore_damier(pstart);
    }

  return r;  
}

small_counter ringxi(void)
{
  int ki,kj;
  small_counter r=0;
  point *pstart,*verystart;
  cell top;

  if (n % 2 == 1)
    return r;
  verystart = &damier[icentre][jcentre];
  set_status(verystart,voisin);
  verystart->next = NULL;
  if (!ring_only)
    r += search(n,verystart);
  set_status(verystart,libre);
  
  for (ki = icentre+1 ; ki < idamier ; ki++)
    for (kj = 0 ; kj < jdamier ; kj++) {
      pstart = &damier[ki][kj];
      start_damier(pstart);
      patch_for_ring(pstart);
      top.car = pstart;
      top.cdr = NULL;
      r += inc_ring(n-2,pstart,pstart,&top);
      unpatch_for_ring(pstart);
      restore_damier(pstart);
    }
 return r;
}

small_counter sym(point *start)
{
  small_counter r=0;
  point *untried;

  untried = start;
  set_status(untried,voisin);
  untried->next = NULL;
  r = search(n,untried);
  set_status(untried,libre);
  return r;
}


void usage(char *pgm)
{
  fprintf(stderr,"usage: %s [-v[pr]]* [n]\n",pgm);
  exit(-1);
}

#define FIXE "fixes.txt"

counter read_fixe(void)
{
  FILE *fp;
  int i;
  counter r;

  fp = fopen(FIXE,"r");
  if (!fp) {
    fprintf(stderr,"Cannot open file ");
    perror(FIXE);
    exit(-1);
  }
  while (fscanf(fp,"%d",&i) == 1) {
    read10(fp,&r);
    if (i == n) {
      fclose(fp);
      return r;
    }
  }
  fprintf(stderr,"Could not find fixed count for %d in file %s\n",n,FIXE);
  exit(-1);
}

int main(int argc,char **argv)
{
  char *arg=0;
  char *pgm = argv[0];
  small_counter rii,rxx,rxi,r2i,r2x,vi,vx,d;
  int except = 0;

  counter r,r2,v,libres;
  counter fixes;
  point *pstart;

  rii = rxx = rxi = r2i = r2x = vi = vx = d = 0;

  while (--argc) {
    arg = *++argv;
    if (*arg == '-') {
      arg++;
      if (*arg == 'v') {
        arg++;
        if (*arg == 'p') {
          verbose_pol++;
        } else if (*arg == 'r') {
          verbose_ring++;
        } else if (*arg == 0) {
          verbose_pol++;
          verbose_ring++;
        } else
          usage(pgm);
      } else if (*arg == 'r') {
        ring_only = 1;
      } else if (*arg == 'e') {
        except = atoi(++arg);
      } else usage(pgm);
    } else {
      break;
    }
  }
  if (argc == 1) {
      n = atoi(arg);
  } else if (argc != 0) {
    usage(pgm);
  }
  if (n > NMAX || n <= 0) {
    fprintf(stderr,"argument must be > 0 and <= %d\n",NMAX);
    exit(2);
  }


  if (! (except & 1)) {
    init_damierii();
    rii = ringii();

    if (n % 2 == 0) {
      init_damierxx();
      rxx = ringxx();

      init_damierxi();
      rxi = ringxi();
    } else {
      rxx = rxi = 0;
    }
  }
  if (! (except & 2)) {
    if (n % 4 == 0 || n % 4 == 1) {
      init_damier2i();
      r2i = ringii();
    } else
      r2i = 0;
  
    if (n % 4 == 0) {
      init_damier2x();
      r2x = ringxx();
    } else
      r2x = 0;
  }

  if (!ring_only && !(except & 4)) {
    init_damiervi();
    pstart = &damier[icentre][jcentre];
    patch_vsym(pstart);
    vi = sym(pstart);
    unpatch_vsym(pstart);

    if (n % 2 == 0) {
      init_damiervx();
      pstart = &damier[icentre][jcentre];
      patch_vsym(pstart);
      vx = sym(pstart);
      unpatch_vsym(pstart);
    } else
      vx = 0;

    init_damierd();
    pstart = &damier[icentre][jcentre];
    patch_dsym(pstart);
    d = sym(pstart);
    unpatch_dsym(pstart);
  } else {
    vx = vi = d = 0;
  }

  printf("  \tRII\tRXX\tRXI\tR2I\tR2X\tVI\tVX\tD\n");
  printf("%2d\t%lu\t%lu\t%lu\t%lu\t%lu\t%lu\t%lu\t%lu\n",n,rii,rxx,rxi,r2i,r2x,vi,vx,d);

  if (ring_only || except)
    return 0;

  r = addi(addi(promote(rii),rxx),2*rxi);
  r2 = addi(promote(r2i),r2x);
  v = addi(promote(vi),vx);

  fixes = read_fixe();

  libres = add(fixes,r);
  libres = add(libres,multi(add(v,addi(r2,d)),2));

  if ((libres.low % 8) != 0) {
    fprintf(stderr,"Warning: strange 8*free count: %lu !!!\n",libres.low % 8);
    
  }
  libres = div2(div2(div2(libres)));

  printf("\n%d\t",n);
  print10(stdout,libres);
  putchar('\t');
  print10(stdout,fixes);
  putchar('\n');
  return 0;
}
